// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package pathfinder

import (
	"reflect"
	"strings"
)

// This function returns the value of a field in a struct, given a path to the field.
func GetValue[D any](obj D, keyWithDots string) (any, error) {
	keySlice := strings.Split(keyWithDots, ".")
	v := reflect.ValueOf(obj)

	for _, key := range keySlice[0 : len(keySlice)-1] {
		for v.Kind() == reflect.Ptr {
			if v.Kind() == reflect.Invalid {
				return nil, newInvalidPathError(keyWithDots)
			}
			v = v.Elem()
		}

		if v.Kind() != reflect.Struct {
			return nil, newUnsupportedTypePathError(keyWithDots, v.Type())
		}

		v = v.FieldByName(key)
	}

	if v.Kind() == reflect.Invalid {
		return nil, newInvalidPathError(keyWithDots)
	}

	for v.Kind() == reflect.Ptr {
		v = v.Elem()
	}

	// non-supporter type at the top of the path
	if v.Kind() != reflect.Struct {
		return nil, newUnsupportedTypeAtTopOfPathError(keyWithDots, v.Type())
	}

	v = v.FieldByName(keySlice[len(keySlice)-1])
	if !v.IsValid() {
		return nil, newInvalidPathError(keyWithDots)
	}

	return v.Interface(), nil

}
